@@ -206,7 +206,7 @@ impl ForeignPredicateDefinitionImpl for SemidetForeignPredicateDefinition {
                 unsafe extern "C" fn #trampoline_name(
                     term: #crt::fli::term_t,
                     arity: std::os::raw::c_int,
-                    _control: *mut std::os::raw::c_void
+                    _control: #crt::fli::control_t
                 ) -> isize {
                     let result = #crt::context::prolog_catch_unwind(|| {
                         if #known_arity as usize != arity as usize {
@@ -253,6 +253,10 @@ impl ForeignPredicateDefinitionImpl for SemidetForeignPredicateDefinition {
             &format!("register_{}", self.predicate_rust_name),
             Span::call_site(),
         );
+        let registration_in_module_name = Ident::new(
+            &format!("register_{}_in_module", self.predicate_rust_name),
+            Span::call_site(),
+        );
         let module_lit = match module {
             None => quote! {None},
             Some(m) => quote! {Some(#m)},
@@ -265,11 +269,11 @@ impl ForeignPredicateDefinitionImpl for SemidetForeignPredicateDefinition {
         let arity = self.params.len() - 1;
 
         quote! {
-            #visibility fn #registration_name() -> bool {
+            #visibility fn #registration_in_module_name(module: Option<&str>) -> bool {
                 // unsafe justification: register_foreign_in_module is unsafe due to the possibility that someone registers a function that is not actually expecting to handle a prolog call, and is not set up right for it. But we're generating this code ourselves, so we should have taken care of all preconditions.
                 unsafe {
                     #crt::init::register_foreign_in_module(
-                        #module_lit,
+                        module,
                         #name_lit,
                         std::convert::TryInto::<u16>::try_into(#arity).expect("arity does not fit in an u16"),
                         true, // deterministic
@@ -278,6 +282,10 @@ impl ForeignPredicateDefinitionImpl for SemidetForeignPredicateDefinition {
                     )
                 }
             }
+
+            #visibility fn #registration_name() -> bool {
+                #registration_in_module_name(#module_lit)
+            }
         }
     }
 
@@ -403,7 +411,7 @@ impl ForeignPredicateDefinitionImpl for NondetForeignPredicateDefinition {
                 unsafe extern "C" fn #trampoline_name(
                     term: #crt::fli::term_t,
                     arity: std::os::raw::c_int,
-                    control: *mut std::os::raw::c_void
+                    control: #crt::fli::control_t
                 ) -> isize {
                     // first, assert that the data type is send and unpin
                     #trampoline_type_check::<#data_type>();
@@ -504,6 +512,10 @@ impl ForeignPredicateDefinitionImpl for NondetForeignPredicateDefinition {
             None => quote! {None},
             Some(m) => quote! {Some(#m)},
         };
+        let registration_in_module_name = Ident::new(
+            &format!("register_{}_in_module", self.predicate_rust_name),
+            Span::call_site(),
+        );
         let rust_name = format!("{}", self.predicate_rust_name);
         let name_lit = match name {
             None => quote! {#rust_name},
@@ -512,11 +524,11 @@ impl ForeignPredicateDefinitionImpl for NondetForeignPredicateDefinition {
         let arity = self.params.len() - 1;
 
         quote! {
-            #visibility fn #registration_name() -> bool {
+            #visibility fn #registration_in_module_name(module: Option<&str>) -> bool {
                 // unsafe justification: register_foreign_in_module is unsafe due to the possibility that someone registers a function that is not actually expecting to handle a prolog call, and is not set up right for it. But we're generating this code ourselves, so we should have taken care of all preconditions.
                 unsafe {
                     #crt::init::register_foreign_in_module(
-                        #module_lit,
+                        module,
                         #name_lit,
                         std::convert::TryInto::<u16>::try_into(#arity).expect("arity does not fit in an u16"),
                         false, // deterministic
@@ -525,6 +537,10 @@ impl ForeignPredicateDefinitionImpl for NondetForeignPredicateDefinition {
                     )
                 }
             }
+
+            #visibility fn #registration_name() -> bool {
+                #registration_in_module_name(#module_lit)
+            }
         }
     }
 
